/********************
Stoffel Kuenen
stoffel@kuenen.org
april 2011 - CC BY-SA 
************************
 - makes 2 motorfaders follow eachothers position
 - uses a basic PD-controller (variant of PID comntroller) for the position control of the faders

 - may be used with MAXMSP patch named: slider_feedback_xx (where xx is a number)
 - there is a 'slow' SetPoint update, which makes the global setpoint slowly follow the setpoint received over Serial
***************************************************
TODO:
  - spongyness in feedback, find sweetspot for PD controller variables between instability and speed
  - delay/spongyness/friction when MAXMSP is attached, find sweetspot between instability and speed in slowSetPointUpdate()
  - implement error detection (correction?) in serial communication
  - smooth sensor data, smooth serial comm data (also in MAXMSP)
  
*********************//*
Specs:
H-bridge: L298N
Vs (H-bridge) = 12Vdc
@MotorSPeed = 255, CurrentSense=~50
Rsense = 0,47 Ohm
************************
motor fader pots connected to Analog0 and Analog1
****************************************************
H-Bridge connected:
                    ENA/B - pin 9,10 (PWM) 
                    1A/B  - pin 8,12  
                    2A/B  - pin 7,13 

pin  10 12  13   motorfader B motor
    ENB 1B  2B

pin  9   8   7   motorfader B motor         
   ENA  1A  2A   function
     H   L   H   Turn Left
     H   H   L   Turn Right
     H   L   L   Fast Motor Stop
     H   H   H   Fast Motor Stop
     L   X   X   Fast Motor Stop

***************************************************/


/*************************************************
****** CREDITS and INSPIRATION *******************
**************************************************
 Changing PWM speeds of the analogWrite() function
**************************************************
 based on:
 http://arduino.cc/en/Tutorial/SecretsOfArduinoPWM
 http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1235060559/4#4
 
 to elaborate: 
 http://smrobots.com/node/22
 http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1238817116
 
 This example shows how to the PWM frequencies resulting from 
 analogWrite() function.
 
 Note:
 - if TCCR0B (timer0) is changed, the millis() and delay() changes
 - TCCR1B (timer1) is a 16 bit timer (?!)
 
Pins 5 and 6:
Setting Divisor Frequency
0x01 	1 	62500        delay(64000) or 64000 millis() ~ 1 second
0x02 	8 	7812.5       delay(8000) or 8000 millis() ~ 1 second
0x03 	64 	976.5625     is the default
0x04 	256 	244.140625   delay(250) or 250 millis() ~ 1 second
0x05 	1024 	61.03515625  delay(62) or 62 millis() ~ 1 second
                             (Or 63 if you need to round up.  The number is actually 62.5)

TCCR0B = TCCR0B & 0b11111000 | <setting>;

Pins 9 and 10:
Setting 	Divisor 	Frequency
0x01 	1 	31250
0x02 	8 	3906.25
0x03 	64 	488.28125
0x04 	256 	122.0703125
0x05 	1024 	30.517578125

TCCR1B = TCCR1B & 0b11111000 | <setting>;

Pins 11 and 3:
Setting 	Divisor 	Frequency
0x01 	1 	31250
0x02 	8 	3906.25
0x03 	32 	976.5625
0x04 	64 	488.28125
0x05 	128 	244.140625
0x06 	256 	122.0703125
0x07 	1024 	30.517578125

TCCR2B = TCCR2B & 0b11111000 | <setting>;

All frequencies are in Hz and assume a 16000000 Hz system clock. 


**************************************************
 PID controller functions
**************************************************
 -  http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1282384853
 -  http://en.wikipedia.org/wiki/PID_controller
***************************************************/
